@@ -39,8 +39,7 @@
 #' @export mcor
 mcor <- function(x, y, use=c("na.or.complete","complete.obs","everything","all.obs")){
 
-    # use <- substr(use[1],1,1);
-    use <- match.arg(use,c("na.or.complete","complete.obs","everything","all.obs"));
+    use <- match.arg(use);
     # everything - returns NA if NA
     # all.obs - returns error if NA
     # complete.obs - NAs are removed, returns an error if nothing is left
@@ -61,6 +60,36 @@ mcor <- function(x, y, use=c("na.or.complete","complete.obs","everything","all.o
         }
     }
 
+    # Check the presence of NAs
+    obsNAy <- any(is.na(y));
+    obsNAx <- any(is.na(x));
+    # if(any(obsNAx) | any(obsNAy)){
+    if(obsNAx || obsNAy){
+        if(use=="everything"){
+            return(returner(1));
+        }
+        else if(use=="all.obs"){
+            returner(2);
+        }
+        else if(any(use==c("na.or.complete","complete.obs"))){
+            # x <- x[!obsNAx & !obsNAy,];
+            # y <- y[!obsNAx & !obsNAy,];
+            dataMatrix <- na.omit(data.frame(y=y,x=x));
+            y <- dataMatrix[[1]];
+            x <- dataMatrix[[-1]];
+
+            # If there are less than 2 obs left...
+            if((!is.null(nrow(x)) && nrow(x)<2) || length(x)<2){
+                if(use=="complete.obs"){
+                    return(returner(1));
+                }
+                else{
+                    returner(2);
+                }
+            }
+        }
+    }
+
     # If it is a factor, just use it
     if(is.factor(x)){
         namesX <- colnames(x);
@@ -119,40 +148,6 @@ mcor <- function(x, y, use=c("na.or.complete","complete.obs","everything","all.o
     }
     obs <- length(y);
 
-    # Check the presence of NAs
-    # obsNAy <- obsNAx <- vector("logical",obs);
-    # obsNAx[] <- FALSE;
-    # for(i in 1:ncol(x)){
-    #     obsNAx[] <- obsNAx[] & is.na(x[,i]);
-    # }
-    # obsNAy[] <- is.na(y);
-    obsNAy <- any(is.na(y));
-    obsNAx <- any(is.na(x));
-    # if(any(obsNAx) | any(obsNAy)){
-    if(obsNAx || obsNAy){
-        if(use=="everything"){
-            return(returner(1));
-        }
-        else if(use=="all.obs"){
-            returner(2);
-        }
-        else if(any(use==c("na.or.complete","complete.obs"))){
-            # x <- x[!obsNAx & !obsNAy,];
-            # y <- y[!obsNAx & !obsNAy,];
-            dataMatrix <- na.omit(cbind(y,x));
-            y <- dataMatrix[,1];
-            x <- dataMatrix[,-1];
-            if(nrow(x)<2){
-                if(use=="complete.obs"){
-                    return(returner(1));
-                }
-                else{
-                    returner(2);
-                }
-            }
-        }
-    }
-
     lmFit <- .lm.fit(x,y);
 
     # Use abs() in order to avoid the rounding issues in R
